@@ -13,8 +13,7 @@ from argparse import _ActionsContainer, _ArgumentGroup, \
                      Action, ArgumentError, RawTextHelpFormatter, SUPPRESS, \
                      _UNRECOGNIZED_ARGS_ATTR, Namespace as BaseNamespace, \
                      ArgumentParser as BaseArgumentParser
-from gettext import gettext as gt
-from os import access, environ, X_OK
+from os import environ
 from os.path import abspath, basename, dirname, splitext
 from stat import S_IXUSR
 try:
@@ -22,15 +21,28 @@ try:
 except ImportError:
     from ConfigParser import ConfigParser, NoOptionError, NoSectionError
 
+from .helpers.constants import PYTHON3
+from .helpers.inputs import user_input
 from .helpers.licenses import *
-from .helpers.types import is_long_opt, is_pos_int, is_short_opt
-from .helpers.utils import PYTHON3, user_input
+from .helpers.data.types import is_executable, is_long_opt, is_pos_int, \
+                                is_short_opt
+from .helpers.text import *
+from .helpers.text import configure_docformat, txt_terminal_render
 from .loglib import logger
 
 
-__all__ = ["gt", "ArgumentParser", "SCRIPTNAME_FORMAT", "SUPPRESS"]
+__all__ = ["ArgumentParser", "DUNDERS", "SCRIPTNAME_FORMAT", "SUPPRESS"]
 
 
+BASE_DUNDERS = ['__author__', '__copyright__', '__credits__', '__license__',
+                '__reference__', '__source__', '__training__']
+DUNDERS = BASE_DUNDERS + [
+    '__date__', '__details__', '__description__', '__doc__', '__docformat__',
+    '__email__', '__examples__', '__functions__', '__maximum_python_version__',
+    '__minimum_python_version__', '__priority__', '__product__', '__script__',
+    '__status__', '__version__',
+]
+
 DEFAULT_MAX_LEN     = 20
 DEFAULT_LST_MAX_LEN = 10
 SCRIPTNAME_FORMAT   = "slugified"
@@ -118,8 +130,10 @@ class _NewSubParsersAction(_SubParsersAction):
         parser = self._parser_class(ArgumentParser.globals_dict, add_help=False,
                                     **kwargs)
         parser.name = name
-        i = parser.add_argument_group(gt("extra arguments"))
-        i.add_argument("-h", "--help", action='help', default=SUPPRESS,
+        i = parser.add_argument_group(txt2title(gt("extra arguments")))
+        i.add_argument("-h", action="usage", prefix="show",
+                       help=gt("show usage message and exit"))
+        i.add_argument("--help", action="help", prefix="show",
                        help=gt("show this help message and exit"))
         # add it to the map
         self._name_parser_map[name] = parser
@@ -129,6 +143,20 @@ class _NewSubParsersAction(_SubParsersAction):
         return parser
 
 
+class _UsageAction(Action):
+    """
+    Custom action for displaying the usage message.
+    """
+    def __init__(self, option_strings, dest=SUPPRESS, help=None):
+        super(_UsageAction, self).__init__(option_strings=option_strings,
+                                           dest=SUPPRESS, default=SUPPRESS,
+                                           nargs=0, help=gt(help))
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        parser.print_usage()
+        parser.exit()
+
+
 class _WizardAction(Action):
     """
     Custom action for triggering the wizard, asking for argument values.
@@ -154,6 +182,7 @@ class _NewActionsContainer(_ActionsContainer):
         self.register('action', 'config', _ConfigAction)
         self.register('action', 'demo', _DemoAction)
         self.register('action', 'extend', _ExtendAction)
+        self.register('action', 'usage', _UsageAction)
         self.register('action', 'wizard', _WizardAction)
     
     def add_argument(self, *args, **kwargs):
@@ -186,7 +215,8 @@ class _NewActionsContainer(_ActionsContainer):
             short_opt = list(filter(is_short_opt, args))
             if len(short_opt) > 0:
                 args.remove(short_opt[0])
-                return self.add_argument(*args, **kwargs)
+                if len(args) > 0:
+                    return self.add_argument(*args, **kwargs)
             # otherwise, retry after modifying the long option string with the
             #  precedence to the prefix (if set) then the suffix (if set)
             long_opt = list(filter(is_long_opt, args))
@@ -261,20 +291,20 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
     name = "main"
     
     def __init__(self, globals_dict=None, *args, **kwargs):
-        self._config_parsed = False
-        self._reparse_args = {'pos': [], 'opt': [], 'sub': []}
         ArgumentParser.globals_dict = gd = globals_dict or {}
-        self.examples = gd.get('__examples__')
-        if self.examples == [] and len(self.examples) == 0:
-            self.examples = None
-        script = gd.get('__file__') or sys.argv[0]
+        configure_docformat(gd)
+        self._config_parsed = False
+        self._docfmt = gd.get('__docformat__')
+        self._reparse_args = {'pos': [], 'opt': [], 'sub': []}
+        self.examples = gd.get('__examples__', [])
+        script = gd.get('__file__', sys.argv[0])
         if script and kwargs.get('prog') is None:
             path = abspath(script)
             root = dirname(path)
             script = basename(script)
             kwargs['prog'] = "python{} ".format(["", "3"][PYTHON3]) + \
-                           script if not access(path, X_OK) else "./" + script \
-                           if root not in environ['PATH'].split(":") else script
+                          script if not is_executable(path) else "./" + script \
+                          if root not in environ['PATH'].split(":") else script
             ArgumentParser.prog = kwargs['prog']
             script, _ = splitext(script)
         kwargs['add_help'] = False
@@ -285,9 +315,11 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
             _ = ["{} {}".format(ArgumentParser.prog, e) for e in self.examples]
             _ = list(filter(lambda x: x.startswith(kwargs['prog']), _))
             if len(_) > 0:
-                kwargs['epilog'] = gt("Usage example{}"
-                                      .format(["", "s"][len(_) > 1])) + \
-                                   ":\n" + '\n'.join("  " + e for e in _)
+                kwargs['epilog'] = txt2title(gt("Usage example{}"
+                                   .format(["", "s"][len(_) > 1])) + ":")
+                e = '\n'.join(["\n", "  "][self._docfmt is None] + \
+                              txt2paragraph(e) for e in _)
+                kwargs['epilog'] += "\n" + e
         # adapt the script name ; if SCRIPTNAME is provided, it supersedes
         #  SCRIPTNAME_FORMAT, otherwise compute the name according to the
         #  format specified in SCRIPTNAME_FORMAT
@@ -301,31 +333,33 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
                 l = "\n- ".join(sorted(SCRIPTNAME_FORMATS.keys()))
                 raise ValueError("Bad script name format ; please use one of "
                                  "the followings:\n{}".format(l))
-        self.scriptname = sname
+        self.scriptname = gd['__scriptname__'] = sname
         # format the description message
         d = sname
-        v = str(gd.get('__version__'))
-        if v:
-            d += " v" + v
+        d += " " + str(gd.get('__version__') or "")
+        d = txt2title(d, level=1)
         v = gd.get('__status__')
         if v:
             d += " (" + v + ")"
-        dunders = ['__author__', '__copyright__', '__license__',
-                   '__reference__', '__source__', '__training__']
-        l = max(list(map(lambda x: len(x.strip('_')), dunders)))
-        for k in dunders:
+        l = max(list(map(lambda x: len(txt2italic(x.strip('_'))),
+                         BASE_DUNDERS)))
+        for k in BASE_DUNDERS:
             m = gd.get(k)
             if m:
                 if k == '__copyright__':
-                    m = copyright(m)
+                    if not isinstance(m, tuple):
+                        m = (m, )
+                    m = copyright(*m)
                 elif k == '__license__':
                     m = license(m, True) or m
-                d += ("\n{: <%d}: {}" % l).format(k.strip('_').capitalize(), m)
+                meta = ("{: <%d}: {}" % l) \
+                       .format(txt2italic(k.strip('_').capitalize()), m)
                 if k == '__author__':
                     e = gd.get('__email__')
                     if e:
-                        d += " ({})".format(e)
-        doc = gd.get('__doc__')
+                        meta += " ({})".format(txt2email(e))
+                d += ["\n\n", "\n"][self._docfmt is None] + txt2paragraph(meta)
+        doc = txt2blockquote(gd.get('__doc__') or "")
         if doc:
             d += "\n\n" + doc
         kwargs['description'] = d
@@ -465,8 +499,7 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
         for a in filter(lambda _: self.is_action(_, 'parsers'),
                         self._actions):
             yield a
-        
-        
+    
     def config_args(self, section="main"):
         """
         Additional method for feeding input arguments from a config file.
@@ -494,34 +527,6 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
         argv = random.choice(self.examples).replace("--demo", "")
         self._reparse_args['pos'] = shlex.split(argv)
     
-    def input_args(self):
-        """
-        Additional method for making the user input arguments manually.
-        """
-        for a in self._sorted_actions():
-            self._set_arg(a)
-        
-    def parse_args(self, args=None, namespace=None):
-        """
-        Reparses new arguments when _DemoAction (triggering parser.demo_args())
-         or _WizardAction (triggering input_args()) was called.
-        """
-        if not namespace:  # use the new Namespace class for handling _config
-            namespace = Namespace(self)
-        namespace = super(ArgumentParser, self).parse_args(args, namespace)
-        if len(self._reparse_args['pos']) > 0 or \
-           len(self._reparse_args['opt']) > 0 or \
-           len(self._reparse_args['sub']) > 0:
-            args = self._reset_args()
-            namespace = super(ArgumentParser, self).parse_args(args, namespace)
-        # process "-hh..." here, after having parsed the arguments
-        help_level = getattr(namespace, "help", 0)
-        if help_level > 0:
-            self.print_help()
-            self.print_extended_help(help_level)
-            self.exit()
-        return namespace
-
     def error(self, message):
         """
         Prints a usage message incorporating the message to stderr and exits in
@@ -535,13 +540,109 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
             self.print_usage(sys.stderr)
             self.exit(2, gt("%s: error: %s\n") % (self.prog, message))
     
+    def format_help(self):
+        text = ""
+        # description
+        formatter = self._get_formatter()
+        formatter.add_text(self.description)
+        text += formatter.format_help() + "\n"
+        # usage
+        text += self.format_usage().rstrip("\n") + "\n\n"
+        # positionals, optionals and user-defined groups
+        for action_group in self._action_groups:
+            formatter = self._get_formatter()
+            formatter.start_section(txt2title(gt(action_group.title) + ":"))
+            formatter.add_text(action_group.description)
+            formatter.add_arguments(action_group._group_actions)
+            formatter.end_section()
+            actions = formatter.format_help()
+            a = ""
+            for i, line in enumerate(actions.splitlines()):
+                s, dedent = line.lstrip(), 0
+                if i == 0:
+                    # action_group.title has ":" added after txt2title(...)
+                    #  causing an extra ":" to be appended
+                    if self._docfmt == "html":
+                        # for markup languages with tags, ":" appears behind
+                        #  the title tag
+                        a += line.rstrip(":") + "\n"
+                    else:
+                        # for other markup languages without tags, ":"
+                        #  appears at the end in duplicate
+                        a += line.rstrip(":") + ":\n"
+                    continue
+                if i == 1 and self._docfmt == "rst":
+                    # action_group.title has ":" added after txt2title(...)
+                    #  causing an extra ":" to be appended behind the
+                    #  underline, making rendering fail
+                    a = a.rstrip("\n") + "\n" + line.rstrip(":") + "\n"
+                    continue
+                if self._docfmt and s[0] in "-*":
+                    dedent = len(line) - len(s)
+                a += (line + "\n\n") if line.rstrip()[-1] == ":" else \
+                     (txt2paragraph(line[dedent:]) + \
+                     ["\n\n", "\n"][self._docfmt is None])
+            text += a + "\n"
+        # epilog
+        formatter = self._get_formatter()
+        formatter.add_text(self.epilog)
+        text += formatter.format_help()
+        # determine help from format above
+        return txt_terminal_render(text)
+    
+    def format_usage(self):
+        formatter = self._get_formatter()
+        formatter.add_usage(self.usage, self._actions,
+                            self._mutually_exclusive_groups)
+        text = formatter.format_help()
+        if self._docfmt:
+            title, usage = text.split(": ", 1)
+            text = txt2title(title + ":") + "\n\n" + \
+                   txt2paragraph(usage.rstrip("\n")) + "\n"
+        return text
+    
+    def input_args(self):
+        """
+        Additional method for making the user input arguments manually.
+        """
+        for a in self._sorted_actions():
+            self._set_arg(a)
+    
+    def parse_args(self, args=None, namespace=None):
+        """
+        Reparses new arguments when _DemoAction (triggering parser.demo_args())
+         or _WizardAction (triggering input_args()) was called.
+        """
+        if not namespace:  # use the new Namespace class for handling _config
+            namespace = Namespace(self)
+        if len(sys.argv) == 2 and sys.argv[1] == "DISPLAY_USAGE":
+            self.print_usage()
+            self.exit()
+        namespace = super(ArgumentParser, self).parse_args(args, namespace)
+        if len(self._reparse_args['pos']) > 0 or \
+           len(self._reparse_args['opt']) > 0 or \
+           len(self._reparse_args['sub']) > 0:
+            args = self._reset_args()
+            namespace = super(ArgumentParser, self).parse_args(args, namespace)
+        # process "-hh..." here, after having parsed the arguments
+        help_level = getattr(namespace, "help", 0)
+        if help_level > 0:
+            self.print_help()
+            self.print_extended_help(help_level)
+            self.exit()
+        return namespace
+    
     def print_extended_help(self, level=1, file=None):
         if not isinstance(self.details, (tuple, list, set)):
             self.details = [self.details]
         for _, message in zip((level - 1) * [None], self.details):
             message = "\n{}\n".format(message.strip())
-            self._print_message(message, file)
-
+            self._print_message(message, file or sys.stdout)
+    
+    def print_usage(self, file=None):
+        self._print_message(txt_terminal_render(self.format_usage()),
+                            file or sys.stdout)
+    
     @classmethod
     def add_to_config(cls, section, name, value):
         """
@@ -613,7 +714,7 @@ class Namespace(BaseNamespace):
         self._collisions = {a.orig: a.dest for a in parser._actions \
                             if getattr(a, "orig", None)}
         self._subparsers = [a.dest for a in parser._filtered_actions("parsers")]
-
+    
     def __getattr__(self, name):
         # handle __privdict__ entry first
         if (name.startswith("_") or name in self.excludes) and \
@@ -621,7 +722,7 @@ class Namespace(BaseNamespace):
             return self.__privdict__[name]
         # then get the attribute the normal way
         return self.__getattribute__(name)
-          
+    
     def __setattr__(self, name, value):
         # handle __privdict__ entry first
         if (name.startswith("_") or name in self.excludes) and \
